<?php

App::uses('EvCheckoutAppModel', 'EvCheckout.Model');
App::uses('CakeEvent', 'Event');
App::uses('AppNonDeletableModel', 'Model');
App::uses('CustomEmail', 'Lib');
App::uses('CakeSession', 'Model/Datasource');
App::uses('CakeEmail', 'Network/Email');
App::uses('OrderStatus', 'EvCheckout.Model');

class Order extends EvCheckoutAppModel {

	public $belongsTo = array(
		'Customer' => array(
			'className' => 'EvCheckout.Customer'
		),
		'DeliveryAddress' => array(
			'className' => 'EvCheckout.CustomerAddress',
			'foreignKey' => 'delivery_address_id'
		),
		'InvoiceAddress' => array(
			'className' => 'EvCheckout.CustomerAddress',
			'foreignKey' => 'invoice_address_id'
		),
		'OrderStatus' => array(
			'className' => 'EvCheckout.OrderStatus'
		)
	);

	public $hasMany = array(
		'OrderItem' => array(
			'className' => 'EvCheckout.OrderItem'
		)
	);

	 public $hasOne = array(
		'Transaction' => array(
			'className' => 'Transactions.Transaction',
			'foreignKey' => 'model_id',
			'conditions' => array(
					'Transaction.model' => 'MODELNAME'
			)
		)
	);


/**
 * Default order of order listings
 */
	public $order = 'Order.created DESC';

/**
 * Temporary variable used to cache the "Order.cart_id" session variable
 * @var int
 */
	private $_cartId = null; // User's shopping cart ID



/**
 * Adds an item to the basket 
 * @param array $data An array of information about the product been added.
 *                    The following params are mandatory 
 *                    $data['order_id'] - The ID of the order to attact it to
 *                    $data['model'] - The model alias of the model the product belongs to 
 *                    $data['model_id'] - The ID of the model 
 * @return boolean	  Returns false if failed else returns true
 */
	public function addToBasket($data) {

		$defaults = array(
			'model' => '',
			'model_id' => '',
			'order_id' => '',
			'name' => '',
			'description' => '',
			'unit_cost' => 0,
			'quantity' => 1,
			'subtotal' => 0,
			'vat' => 0,
			'total' => 0,
			'is_virtual' => false,
		);
		$data = array_merge($defaults, $data);
		
		//Check required fields are filled in
		if (empty($data['order_id']) ||  empty($data['model_id']) ||  empty($data['model'])) {
			return false;
		}

		$data['subtotal'] = $data['quantity'] * $data['unit_cost'];
		
		//Send off event to allow listeners to manipulate the order item before save
		$event = new CakeEvent('Model.Order.addToBasket', $this, array(
			'OrderItem' => $data
		));
		$this->getEventManager()->dispatch($event);
		$data = $event->data;

		if($this->OrderItem->save($data)) {

			$this->updateOrderTotal();

		} else {

			return false; 

		}

		return true;

	}


/**
 * Sets the User of the given cart or session cart 
 * @param int $userId User::id
 * @param int $cartId The ID of the cart
 */
	public function setUser($userId, $cartId = false) {

		$this->id = ($cartId) ? $cartId : $this->getCartID();

		$this->saveField('user_id', $userId);

	}


/**
 * Gets Cart ID from SESSION 
 * @param  boolean  Creates a new cart if there isn't one currently
 * @return int Cart/Order ID
 */
	public function getCartId($createIfNotExists = false) {

		if (!isset($this->_cartId) || $this->_cartId==null) {

			$this->_cartId = CakeSession::read('Order.cart_id');

		}

		//Cannot find cart so create a new one 
		if(! $this->_cartId) {

			$this->createOrder();

		}


		return $this->_cartId;
	}


/**
 * Clears the cart session ID used when the order has been completed
 * @return null
 */
	public function clearCartId() {

		$this->_cartId = null;
		CakeSession::write('Order.cart_id', $this->_cartId);

	}


/**
 * Gets the current Order item either by the passed ID or by the Session variable using Order::getCartId
 * @param  mixed $id      The ID of the cart
 * @param  boolean  $isAdmin Indicates whether the method is being accessed from the admin 
 * @return array          Find first of the Order array or false if no cart can be found
 */
	public function getCart($id=false, $isAdmin = false) {

		//If no $id passed find from session
		$id = ($id) ? $id : $this->getCartId();

		// assign order status check, the unpaid status check is not required when
		// method is called from the admin
		if (!$isAdmin) {

			$conditions = array($this->alias.'.id' => $id, $this->alias.'.order_status_id' => OrderStatus::IN_BASKET);

		} else {

			$conditions =  array($this->alias.'.id' => $id);

		}

		$order = $this->find('first', array(
			'conditions' => $conditions,
			'contain' => array(
				'DeliveryAddress',
				'InvoiceAddress',
				'OrderItem'
			)
		));

		// Clear cart if user has already paid for order or it has been deleted
		if (empty($order)) {

			$this->clearCartId();

			return false;
		}

		return $order;

	}

/**
 * Creates a new cart and saves the ID into the session
 * @return int The ID of the new cart
 */
	public function createOrder() {

		$data = array(
			'order_status_id' => OrderStatus::IN_BASKET
		);

		$this->create();
		$this->save($data);

		$this->_cartId = $this->id;

		CakeSession::write('Order.cart_id', $this->_cartId);

		return $this->_cartId;

	}



	public function updateOrderTotal() {

		$order = $this->getCart();

		$subtotal = 0.00;
		$vat = 0.00;

		if (isset($order['OrderItem'])) {
			
			foreach ($order['OrderItem'] as $item) {

				$subtotal += $item['subtotal'];
				$vat += $item['vat'];

			}

		}

		$productsTotal = $total = $subtotal;

		$data = array(
			'Order' => array(
				'id' => $order['Order']['id'],
				'delivery_cost' => '0', //Set by Event dispatch later in the method
				'vat' => $vat,
				'subtotal' => $subtotal,
				'total' => $total,
				'items_total' => $productsTotal,
				'order_time' => date('Y-m-d H:i:s'),
				'delivery_surcharge' => '0',  //Set by Event dispatch later in the method
			)
		);

		//Assign user if there is a signed in one
		$userId = CakeSession::read('Auth.User.User.id');
		if (!empty($userId)) {
			$data['Order']['user_id'] = $userId;
		}

		// Delivery, discounts etc, calc'd in event
		$event = new CakeEvent('Model.Order.updateOrderTotal', $this, array(
				'Order' => array_merge($order, $data)
			)
		);
		$this->getEventManager()->dispatch($event);
		$data = $event->data['Order'];

		//Return total if success else false
		return ($this->save($data) ? $total : false);
	}

	
	public function setDeliveryAddress($id = null) {

		$deliveryAddress = null;

		$this->id = $this->getCartId();

		if ($id) {

			$this->DeliveryAddress->id = $id;
			$this->DeliveryAddress->saveField('last_used_for_delivery', date('Y-m-d H:i:s'));

			$deliveryAddress = $this->saveField('delivery_address_id', $id);

		}

		return $deliveryAddress;
	}

	public function setInvoiceAddress($id) {

		$this->id = $this->getCartId();
		if ($id) {
			$this->InvoiceAddress->id = $id;
			$this->InvoiceAddress->saveField('last_used_for_invoice', date('Y-m-d H:i:s'));
		}

		return $this->saveField('invoice_address_id', $id);

	}



	// protected function _incrementUnitsSold($order, $isAdmin = null) {

	// 	// if no order items are passed then attempt to retreive them
	// 	// * the call made from afterSave will contain no order items
	// 	if (empty($order['Items'])) {

	// 		if (!empty($order['Order']['id'])) {

	// 			$order = $this->getCart($order['Order']['id'], $isAdmin);

	// 		}

	// 	}

	// 	// increment each product puchased within the order
	// 	if (!empty($order['Items'])) foreach ($order['Items'] as $item) {

	// 		if (!empty($item['OrderItem']['product_id']) && !empty($item['OrderItem']['quantity'])) {

	// 			$this->query("UPDATE `products` AS `Product` SET `Product`.`units_sold` = `Product`.`units_sold`+" . $item['OrderItem']['quantity'] . " WHERE `Product`.`id` =" . $item['OrderItem']['product_id']);

	// 		}

	// 	}

	// 	return;

	// }


	public function markAsPaid($id=false, $success=true, $method = false) {

		$id = $id ? $id : $this->getCartId();

		$order = $this->getCart($id);

		// Only mark as paid once
		if (isset($order['Order']['status']) && $order['Order']['status'] != self::PAID) {

			if ($success) {

				$data = array(
					'Order' => array(
						'id' => $id,
						'invoice_id' => $this->getNextInvoiceNumber(),
						'status' => self::PAID,
						'order_time' => date('Y-m-d H:i:s'),
						'payment_method' =>  $method,
					)
				);

				$this->save($data);

				$this->_incrementUnitsSold($order);

				$this->emailConfirmation($id);


			} else {

				$data = array(
					'Order' => array(
						'id' => $id,
						'status' => self::FAILED,
						'order_time' => date('Y-m-d H:i:s')
					)
				);

				$this->save($data);

			}

			$this->clearCartId();

		}
	}

	public function emailConfirmation($id, $delayed = false) {

		$order = $this->readForEdit($id);

		if (!empty($order)) {

			$email = new CustomEmail();

			$email->template('confirmation');

			$email->emailFormat('html');

			$email->to($order['User']['email']);
			
			$this->_configureConfirmationEmail($email, $order, $delayed);

			$email->helpers(
				array(
					'Html'
				)
			);
			
			return $email->send();
		}

	}


	protected function _configureConfirmationEmail(&$email, $order, $delayed) {
		$email->from(array(EmailConfig::SYSTEM_EMAIL => Configure::read('SiteSetting.site_title')));
		$email->subject(Configure::read('SiteSetting.site_title') . ' Order Confirmation');

		$email->viewVars(
			array(
				'order' => $order,
				'delayed' => $delayed,
				'telephone' => Configure::read('SiteSetting.telephone_number')
			)
		);
		// $email->addBcc(Configure::read('SiteSetting.sales_email'));
		// $email->addBcc("8b6fcba9@trustpilotservice.com");
	}



	public function isCartVirtual() {
		$cart = $this->getCart();
		foreach ($cart['Items'] as $item) {
			if (!$item['Product']['is_virtual'])
				return false;
		}
		return true;
	}

}